#include "main.h"
#include "board.h"
#include "configuration.h"
#include "button.h"
#include "packet.h"

//extern NAND_HandleTypeDef hnand1;
BUTTONInfo_TypeDef BUTTON[MAX_NUM_OF_BUTTONS];
#define BUTTON_STATE(b) HAL_GPIO_ReadPin(BUTTON[b].GPIO_Port, BUTTON[b].GPIO_Pin)

void BUTTON_Init(GPIO_TypeDef* GPIOx, uint16_t GPIO_Pin, uint8_t button)
{
	if (button >= MAX_NUM_OF_BUTTONS) return;
	BUTTON[button].GPIO_Port = GPIOx;
	BUTTON[button].GPIO_Pin = GPIO_Pin;
	BUTTON[button].Timer = 0;
	BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
	BUTTON[button].Press = BUTTON_NONE;
}

BUTTONPress_TypeDef BUTTON_KeyPressed(uint8_t button)
{
	BUTTONPress_TypeDef tmp = BUTTON[button].Press;
	if  (BUTTON[button].Press != BUTTON_NONE) BUTTON[button].Press = BUTTON_NONE;
	return tmp;
}
void BUTTON_Handler(void)
{
	uint8_t i;
	//int16_t acc[3] = {0, 0, 0};
	//NAND_IDTypeDef NAND_ID;

	for (i=0; i<MAX_NUM_OF_BUTTONS; i++)
	{
		switch(BUTTON_KeyPressed(i))
		{
		case BUTTON_NONE:
			/* DO NOTHING */
			break;
		case BUTTON_LONGPRESS_DETECTED:
			LED_SetState(LED_YELLOW, 0x0F);
			printf("LONGPRESS DETECTED\n");
			break;
		case BUTTON_LONGPRESS:
			printf("LONGPRESS\n");
			//SLEEPMODE_GotoSTOP2();
			break;
		case BUTTON_DOUBLE_CLICK:
			printf("DOUBLECLICK\n");
			LED_SetState(LED_GREEN, 0x01);
			PACKET_Create(PACKET_MASK_DCLICK);
			break;
		case BUTTON_CLICK:
			LED_SetState(LED_GREEN, 0x01);
			printf("---( CLICK-INFO )---------------------\n");
			PACKET_Create(PACKET_MASK_CLICK);
			break;
		}
	}//for
}

void BUTTON_TimerHandler(void)
{
	uint8_t button = 0;
	for (button=0; button<MAX_NUM_OF_BUTTONS; button++)
	{
		if (BUTTON[button].Timer) BUTTON[button].Timer--;
		switch(BUTTON[button].State)
		{
			case BUTTON_WAIT_TO_PRESS:
				if (BUTTON_STATE(button) == BUTTON_IS_PRESSED)
				{
					BUTTON[button].State = BUTTON_WAIT_TO_LONGPRESS;
					BUTTON[button].Timer = BUTTON_LONGPRESS_IN_MS;
				}
				break;
			case BUTTON_WAIT_TO_LONGPRESS:
				if (BUTTON[button].Timer == 0)
				{
					/*LONG PRESS TIMER IS EXPIRED */
					BUTTON[button].State = BUTTON_WAIT_TO_RELEASE;
					BUTTON[button].Press = 	BUTTON_LONGPRESS_DETECTED;

				}
				if (BUTTON_STATE(button) != BUTTON_IS_PRESSED)
				{
					/* BUTTON IS RELEASED */
					BUTTON[button].State = BUTTON_WAIT_TO_CLICK;
					BUTTON[button].Timer = BUTTON_DOUBLECLICK_WAIT_IN_MS;
				}
				break;
			case BUTTON_WAIT_TO_CLICK:
				if (BUTTON[button].Timer == 0)
				{
					/* CLICK FINISHED*/
					BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
					BUTTON[button].Press = 	BUTTON_CLICK;
	#if defined(BUTTON_DEBUG)
					printf("CLICK\n");
	#endif
				}

				if (BUTTON_STATE(button) == BUTTON_IS_PRESSED)
				{
					/* DOUBLE CLICK IS DETECTED */
					BUTTON[button].State = BUTTON_WAIT_TO_DOUBLECLICK;
				}
				break;
			case BUTTON_WAIT_TO_DOUBLECLICK:
				if (BUTTON_STATE(button) != BUTTON_IS_PRESSED)
				{
					/* DOUBLE CLICK FINISHED*/
					BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
					BUTTON[button].Timer = 0;
					BUTTON[button].Press = 	BUTTON_DOUBLE_CLICK;
	#if defined(BUTTON_DEBUG)
					printf("DOUBLE CLICK\n");
	#endif
				}
				break;
			case BUTTON_WAIT_TO_RELEASE:
				if (BUTTON_STATE(button) != BUTTON_IS_PRESSED)
				{
					BUTTON[button].State = BUTTON_WAIT_TO_PRESS;
					BUTTON[button].Timer = 0;
					BUTTON[button].Press = 	BUTTON_LONGPRESS;
	#if defined(BUTTON_DEBUG)
					printf("LONG PRESS\n");
	#endif
				}
				break;
		}
	}//for
}
