#include "nb.h"

#include <stdio.h>
#include <stm32l4xx.h>
#include <stm32l4xx_hal_uart.h>
#include <string.h>
#include <sys/_stdint.h>

#include "board.h"
#include "buffer.h"
#include "cmdline.h"
#include "mram.h"
#include "packet.h"

#define NB_RX_TIMEOUT_IN_MS 500
uint32_t NBTimeout = 0, NB10sTimer = 10000, NBReceiveIndex=0, NBRxTimeout = NB_RX_TIMEOUT_IN_MS;
Buffer_TypeDef NB_RXMSG;
BufferItem_TypeDef NB_Msg;
NB_ATCommandState PrevState=NB_CMD_NONE;	/* current state of command */
uint8_t rxNB; /* receive data from NB */

NBInfo_TypeDef NBInfo;

NB_ErrorType NB_ATCommand_Handler(void);
ATCommand_TypeDef ATCommands[] = {
		{NB_AT, {"at"}, {""}, NB_CMD_NONE, 3, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CCID, {"at+qccid"}, {"+QCCID: %s"}, NB_CMD_NONE, 2, 3000, NBInfo.ICCID, {""}, },
		{NB_AT_CGSN, {"at+cgsn=1"}, {"+CGSN: %s"}, NB_CMD_NONE, 2, 2000, NBInfo.IMEI, {""}, AT_COMPLETE_NONE},
		{NB_AT_CFUN0, {"at+cfun=0"}, {""}, NB_CMD_NONE, 1, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CFUN1, {"at+cfun=1"}, {""}, NB_CMD_NONE, 1, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_QBAND, {"at+qband=1,20"}, {""}, NB_CMD_NONE, 1, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_COPS, {"at+cops=1,2,\"23003\""}, {""}, NB_CMD_NONE, 1, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CESQ, {"at+cesq"}, {"+CESQ: %d"}, NB_CMD_NONE, 2, 2000, &NBInfo.RxLevel, {""}, AT_COMPLETE_NONE},
		{NB_AT_CIMI, {"at+cimi"}, {"%s"}, NB_CMD_NONE, 2, 20000, NBInfo.IMSI, {""}, AT_COMPLETE_NONE},
		{NB_AT_CREGW, {"at+cereg=5"}, {""}, NB_CMD_NONE, 2, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CREG, {"at+cereg?"}, {"+CEREG: 5,5,%s"}, NB_CMD_NONE, 10, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CGATT, {"at+cgatt?"}, {"+CGATT: %s"}, NB_CMD_NONE, 100, 2000, NULL, {"1"}, AT_COMPLETE_NONE},
		{NB_AT_CSCON, {"at+cscon=0"}, {""}, NB_CMD_NONE, 100, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_ATQIOPEN, {"at+qiopen=1,0,\"UDP\",192.168.0.20,4242,0,0,0"}, {"+QIOPEN: %d"}, NB_CMD_NONE, 10, 2000, &NBInfo.ConnectID, {""}, AT_COMPLETE_NONE},
		{NB_ATQISENDEX, {"at+qisendex="}, {""}, NB_CMD_NONE, 1, 2000, NULL, {""}, AT_COMPLETE_QISENDEX},
		{NB_ATQICLOSE, {"at+qiclose=0"}, {""}, NB_CMD_NONE, 10, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_ATE0, {"ATE0"}, {""}, NB_CMD_NONE, 3, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CGPADDR, {"at+cgpaddr=1"}, {"+CGPADDR: 1,%s"}, NB_CMD_NONE, 10, 2000, NBInfo.IP, {""}, AT_COMPLETE_NONE},
		{NB_AT_QCGDEFCONT, {"at+qcgdefcont=\"IP\",\"nb.m2mc\""}, {""}, NB_CMD_NONE, 1, 2000, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_QPOWD0, {"at+qpowd=0"}, {""}, NB_CMD_NONE, 3, 400, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_CPSMS1, {"at+cpsms=1,,,\"00000001\",\"00000011\""}, {""}, NB_CMD_NONE, 3, 400, NULL, {""}, AT_COMPLETE_NONE},
		{NB_AT_QSCLK2, {"AT+QSCLK=2"}, {""}, NB_CMD_NONE, 3, 800, NULL, {""}, AT_COMPLETE_NONE},

};


NB_ATCommandType NB_Sequence_Info[] = {NB_AT, NB_ATE0, NB_AT_QSCLK2, NB_AT_CGSN, NB_AT_CCID, NB_AT_CREGW, NB_AT_CSCON, NB_AT_QBAND, NB_AT_CFUN0, NB_AT_QCGDEFCONT, NB_AT_CFUN1, NB_AT_CREG, NB_AT_ENDFLAG};
NB_ATCommandType NB_Sequence_Send[] = {NB_AT_CESQ, NB_ATQISENDEX, /*NB_ATQICLOSE, */NB_AT_ENDFLAG};
NB_ATCommandType NB_Sequence_Connect[] = {NB_AT_CGATT, NB_AT_CESQ, NB_ATQIOPEN, NB_AT_CPSMS1, NB_AT_ENDFLAG};
NB_ATCommandType NB_Sequence_Disconnect[] = {NB_ATQICLOSE, NB_AT_ENDFLAG};
NB_ATCommandType NB_Sequence_Wait2IP[] = {NB_AT_CGATT, NB_AT_CGPADDR, NB_AT_CESQ, NB_AT_ENDFLAG};
NB_ATCommandType NB_Sequence_PowerOFF[] = {NB_AT_QPOWD0, NB_AT_ENDFLAG};

void NB_TransmitData(void);
void NB_TransmitIMEI(char *imei);
NB_ErrorType NB_DoAtComplete(NB_ATCommandCompleteType complete);
uint16_t NB_GetDestPort(void);
uint32_t NB_GetDestIP(void);
uint16_t NB_GetSrcPort(void);

void NB_Init(void)
{
	BUFFER_Create(&NB_RXMSG, 32);	//gsm message buffer
	CmdLine_RegisterCmd("nb", "s", NB_DBG, 0);// register debug command
	CmdLine_RegisterCmd("nbi", "", NB_DBG, 1);// register debug command
	CmdLine_RegisterCmd("set", "ss", NB_DBG, 2);// register debug command
	/* Initialize pins to default configuration */
	//BSP_NB_EN_LOW();
	//BSP_NB_RST_LOW();
	NBInfo.Mode = NB_OFF;
	NBInfo.PrevMode = NB_STOP;
	NBInfo.Sequence = NULL;
	NBInfo.Command.State = NB_CMD_NONE;
	NBInfo.IMEI[0] = 0;
	NBInfo.ConnectID = 0xff;
	sprintf(NBInfo.IMEI, "%s", "867997030054596");
	HAL_UART_Receive_IT(&NB_UART_BUS_HANDLER, &rxNB, 1);
}
/*
 * This routine decrement timout timer for NB purposes
 */
void NB_TimerHandler(void)
{
	if (NBTimeout) NBTimeout--;
	if (NBInfo.Command.Timeout) NBInfo.Command.Timeout--;
	if (NB10sTimer) NB10sTimer--;
	if (NBRxTimeout) NBRxTimeout--;
}
FlagStatus NB_10sExpired(void)
{
	if (NB10sTimer == 0)
	{
		NB10sTimer = 1000;
		return SET;
	}
	return RESET;
}

void NB_Start(void)
{
	NBInfo.Mode = NB_START;
}

uint8_t NB_SeqID2Index(NB_ATCommandType id)
{
	uint8_t i=0;
	while (ATCommands[i].Index != NB_AT_ENDFLAG)
	{
		if (ATCommands[i].Index == id) return i;
		i++;
	}
	return 0;
}

void NB_Handler(void)
{
	NB_StateType mode=NBInfo.Mode;
	if (NBTimeout) return;
	if (NBInfo.PrevMode != NBInfo.Mode) printf("NB MODE=%02d\r\n", NBInfo.Mode);
	switch(NBInfo.Mode)
	{
	case NB_OFF:
		NB_ATCommand_Handler();
		NBTimeout = 0;
		if (BUFFER_IsEmpty(&PACKET_TXMSG)) NBInfo.Mode = NB_OFF;
		else NBInfo.Mode = NB_START;
		break;
	case NB_START:
		NBTimeout = 0;	//nothing to wait
		NBInfo.Mode = NB_ENABLE_POWER;
		break;
	case NB_ENABLE_POWER:
		NBTimeout = 100;	//wait to enable power
		NBInfo.Mode = NB_PWRKEY_ON_PULSE;
		BSP_NB_EN_HIGH();
		BSP_NB_PWRKEY_HIGH();	//set PWRKEY to default state
		break;
	case NB_PWRKEY_ON_PULSE:
		NBTimeout = 600;	//POWERKEY is required to be low >500ms
		BSP_NB_PWRKEY_LOW();
		NBInfo.Mode = NB_PWRKEY_OFF_PULSE;
		break;
	case NB_PWRKEY_OFF_PULSE:
		BSP_NB_PWRKEY_HIGH();	//set PWRKEY to default state
		printf("Device is running.\r\n");
		NBInfo.Mode = NB_INFO;
		break;
	case NB_INFO:
		printf("Info Sequence Started\r\n");
		NBInfo.Mode = NB_ON;
		NBInfo.Sequence = NB_Sequence_Info;
		NBInfo.SequenceIndex = 0;
		NBInfo.SequenceDone = RESET;
		NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
		break;
	case NB_ON:
		if (NB_ATCommand_Handler() == NB_ERROR)
		{
			NBInfo.Mode = NB_STOP;
			break;
		}
		if (NBInfo.SequenceDone == SET)
		{
			NBInfo.Sequence = NB_Sequence_Connect;
			NBInfo.SequenceIndex = 0;
			NBInfo.SequenceDone = RESET;
			NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
			NBInfo.Mode = NB_WAITING_TO_CONNECT;
		}
		break;
	case NB_WAITING_TO_CONNECT:
		if (NB_ATCommand_Handler() == NB_ERROR)
		{
			NBInfo.Mode = NB_STOP;
			break;
		}
		if (NBInfo.SequenceDone == SET)
		{
			NBInfo.Sequence = NB_Sequence_Wait2IP;
			NBInfo.SequenceIndex = 0;
			NBInfo.SequenceDone = RESET;
			NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
			NBInfo.Mode = NB_WAITING_TO_IP;
		}
		break;
	case NB_WAITING_TO_IP:
			if (NB_ATCommand_Handler() == NB_ERROR)
			{
				NBInfo.Mode = NB_STOP;
				break;
			}
			if (NBInfo.SequenceDone == SET)
			{
				if (strcmp((char*)&NBInfo.IP, "0") == 0)
				{
					NBInfo.Sequence = NB_Sequence_Wait2IP;
					NBInfo.SequenceIndex = 0;
					NBInfo.SequenceDone = RESET;
					NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
					NBInfo.Mode = NB_WAITING_TO_IP;
					NBTimeout = 1000;	//wait to next step
				}
				else NBInfo.Mode = NB_CONNECTED;
			}
			break;
	case NB_CONNECTED:
		if (NB_ATCommand_Handler() == NB_ERROR)
		{
			NBInfo.Mode = NB_DISCONNECTING;
			NBInfo.Sequence = NB_Sequence_Disconnect;
			NBInfo.SequenceIndex = 0;
			NBInfo.SequenceDone = RESET;
			NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
			NBTimeout = 1000;	//wait to transmit
			break;
		}
		if (NBInfo.SequenceDone == SET)
		{
			/*if (BUFFER_IsEmpty(&PACKET_TXMSG))
			{
				NBInfo.Mode = NB_STOP;
				NBTimeout = 1000;	//wait to transmit
			}*/
			//if (NB_10sExpired() == SET)
			//else
			if (!BUFFER_IsEmpty(&PACKET_TXMSG))
			{
				NBInfo.Sequence = NB_Sequence_Send;
				NBInfo.SequenceIndex = 0;
				NBInfo.SequenceDone = RESET;
				NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
				NBTimeout = 1000;	//wait to transmit
			}
		}
		break;
	case NB_DISCONNECTING:
		if (NB_ATCommand_Handler() == NB_ERROR)
		{
			NBInfo.Mode = NB_STOP;
			break;
		}
		if (NBInfo.SequenceDone == SET)
		{
			NBInfo.Mode = NB_STOP;
		}
		break;
	case NB_STOP:
		NBInfo.Sequence = NB_Sequence_PowerOFF;
		NBInfo.SequenceIndex = 0;
		NBInfo.SequenceDone = RESET;
		NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
		NBTimeout = 1100;	//wait to transmit
		NBInfo.Mode = NB_DISABLE_POWER;
		break;
	case NB_DISABLE_POWER:
		if (NB_ATCommand_Handler() == NB_ERROR)
		{
			NBInfo.Mode = NB_OFF;
			break;
		}
		NBTimeout = 100;
		if (NBInfo.SequenceDone == SET) NBInfo.Mode = NB_OFF;
		break;
	}
	NBInfo.PrevMode = mode;
}

void NB_ATCommandSend(NB_ATCommandType index)
{
	uint8_t cmdindex = NB_SeqID2Index(index);
	uint8_t enter=1;

	NBInfo.Command.Index = index;
	if (NBInfo.Command.State == NB_CMD_TXREPEAT) NBInfo.Command.Retries--;
	else NBInfo.Command.Retries = ATCommands[cmdindex].Retries;
	NBInfo.Command.Timeout = ATCommands[cmdindex].Timeout;
	NBInfo.Command.Value = ATCommands[cmdindex].Value;
	NBInfo.Command.CommandComplete = ATCommands[cmdindex].CommandComplete;
	strcpy(&NBInfo.Command.Command[0], &ATCommands[cmdindex].Command[0]);
	strcpy(&NBInfo.Command.Parse[0], &ATCommands[cmdindex].Parse[0]);
	strcpy(&NBInfo.Command.Check[0], &ATCommands[cmdindex].Check[0]);
	NBInfo.CheckError = 0;
	if (NBInfo.Command.CommandComplete == AT_COMPLETE_NONE)
	{
		printf("%s--->", &NBInfo.Command.Command[0]);
		NBInfo.Command.State = NB_CMD_RXWAIT;
	}
	else
	{
		printf("%s", &NBInfo.Command.Command[0]);
		NBInfo.Command.State = NB_CMD_WAIT_TO_COMPLETE_COMMAND;
		enter = 0;
	}
	NB_SendData((uint8_t*)&NBInfo.Command.Command[0], strlen(NBInfo.Command.Command), enter);
}

NB_ErrorType NB_ATCommand_Handler(void)
{
	BufferItem_TypeDef msg;
	char txt[128];
	uint8_t txtlen;
	uint32_t i;
	//if (NBInfo.Mode == NB_OFF) NBInfo.Mode = NB_START;
	if ((NBRxTimeout == 0) && (NBReceiveIndex))
	{
		printf("#%d#\r\n", NBReceiveIndex);
		NB_Msg.data[NBReceiveIndex] = '\0';
		NB_Msg.len = NBReceiveIndex;
		BUFFER_Add(&NB_RXMSG, &NB_Msg);
		NBReceiveIndex = 0;
		//NBRxTimeout = NB_RX_TIMEOUT_IN_MS;
		return NB_OK;
	}
	switch(NBInfo.Command.State)
	{
	case NB_CMD_NONE:
		if(BUFFER_Get(&NB_RXMSG, &msg) != 0xFF)
		{
			printf("%s\r\n", msg.data);
		}
		break;
	case NB_CMD_TXREPEAT:
		NB_ATCommandSend(NBInfo.Command.Index);
		break;
	case NB_CMD_RXCHECKERROR:
		if (NBInfo.Command.Timeout == 0) NBInfo.Command.State = NB_CMD_RXTIMEOUT;
		break;
	case NB_CMD_RXTIMEOUT:
		if (NBInfo.Command.Index == NB_AT_CIFSR)	//CIFSR not returning OK
		{
			NBInfo.Command.State = NB_CMD_DONE;
			break;
		}
		if (NBInfo.Command.Index == NB_AT_CIPSEND)	//NB_AT_CIPSEND not returning OK
		{
			NBInfo.Command.State = NB_CMD_DONE;
			break;
		}
		if (NBInfo.Command.Retries)
		{
			NBInfo.Command.State = NB_CMD_TXREPEAT;
		}
		else
		{
			printf("TIMEOUT\r\n");
			NBInfo.Command.State = NB_CMD_DONE;
			return NB_ERROR;
		}

		break;
	case NB_CMD_DONE_WITH_OK:
	case NB_CMD_DONE_WITHOUT_OK:
		NBInfo.Command.State = NB_CMD_DONE;
		break;
	case NB_CMD_RXWAIT_TO_ANSWER:
		if (NBInfo.Command.Timeout == 0) NBInfo.Command.State = NB_CMD_RXTIMEOUT;
		break;
	case NB_CMD_WAIT_TO_COMPLETE_COMMAND:
		NB_DoAtComplete(NBInfo.Command.CommandComplete);
		NBInfo.Command.State = NB_CMD_RXWAIT;
		break;
	case NB_CMD_RXWAIT:
		if (NBInfo.Command.Timeout == 0) NBInfo.Command.State = NB_CMD_RXTIMEOUT;
		if(BUFFER_Get(&NB_RXMSG, &msg) != 0xFF)
		{
#if defined (NB_DEBUG)
			printf("<%s>-%d:", msg.data, msg.len);
			for (i=0; i< msg.len; i++) printf(" %02x", msg.data[i]);
			printf("\r\n");
#endif


			printf("%s\r\n", msg.data);
			if (strcmp((char*)&msg.data[0], "OK") == 0)
			{
				if (NBInfo.Command.Index == NB_ATQIOPEN)	return NB_OK;//AT+QIOPEN returning OK and result after that
				if (NBInfo.CheckError)
				{
					NBInfo.Command.State = NB_CMD_RXCHECKERROR;
					return NB_OK;
				}
				NBInfo.Command.State = NB_CMD_DONE;
			}
			else if (strcmp((char*)&msg.data[0], "ERROR") == 0)
			{
				if (NBInfo.Command.Retries == 0) return NB_ERROR;
			}
			else if (strcmp((char*)&msg.data[0], (char*)&NBInfo.Command.Command[0]) == 0)
			{
				//Ignore Echo
			}
			else
			{
				memset(txt, 0, sizeof(txt));
				if (sscanf((char *)&msg.data[0], (char*)&NBInfo.Command.Parse[0], txt) == 1)
				{
					if (NBInfo.Command.Value != NULL)	strcpy(NBInfo.Command.Value, txt);
					if (strcmp(NBInfo.Command.Check, "") && strcmp(txt, NBInfo.Command.Check)) NBInfo.CheckError = 1;
					if (NBInfo.Command.Index == NB_ATQIOPEN)	NBInfo.Command.State = NB_CMD_DONE;
				}
			}
		}
		break;
	case NB_CMD_DONE:
		if (NBInfo.Sequence != NULL)
		{
			if (NBInfo.Sequence[NBInfo.SequenceIndex+1] == NB_AT_ENDFLAG)
			{
				NBInfo.Command.State = NB_CMD_NONE;
				NBInfo.SequenceDone = SET;
				NBInfo.Sequence = NULL;
			}
			else
			{
				NBInfo.SequenceIndex++;
				NB_ATCommandSend(NBInfo.Sequence[NBInfo.SequenceIndex]);
			}
		}
		break;
	}

return NB_OK;
}

NB_ErrorType NB_DoAtComplete(NB_ATCommandCompleteType complete)
{
	BufferItem_TypeDef msg;
	char txt[128]={0};
	uint8_t txtlen;
	uint32_t i, ip;
	switch (complete)
	{
	case AT_COMPLETE_QISENDEX:
		if (BUFFER_Get(&PACKET_TXMSG, &msg) != 0xFF)
		{
			//Add current ConnectID
			txtlen = sprintf(txt, "%d,", NBInfo.ConnectID);
			//Add length of message
			txtlen += sprintf(&txt[txtlen], "%d,", (int)msg.len);
			for (i=0; i< msg.len; i++) txtlen += sprintf(&txt[txtlen], "%02x", msg.data[i]);
			printf("%s--->", txt);
			NB_SendData(txt, txtlen, 1); //nothing to send enter
		}
		break;
	case AT_COMPLETE_NONE:
		break;
	}
	return NB_OK;
}

void NB_SaveByte(uint8_t* ucByte)
{
	if ((*ucByte == '\r') || (NB_Msg.data[NBReceiveIndex-1] == '\r'))	//<CR><LF> detected
	//if (*ucByte == '\r')
	{
		if (NBReceiveIndex)
		{
			NB_Msg.data[NBReceiveIndex] = '\0';
			NB_Msg.len = NBReceiveIndex;
			BUFFER_Add(&NB_RXMSG, &NB_Msg);
			NBReceiveIndex = 0;
		}
		//NBReceiveIndex = 0;
	}
	else if (*ucByte == '\n')
	{
		//ignore LF
	}
	else
	{
		NB_Msg.data[NBReceiveIndex++] = *ucByte;

	}
	NBRxTimeout = NB_RX_TIMEOUT_IN_MS;
	HAL_UART_Receive_IT(&NB_UART_BUS_HANDLER, &rxNB, 1);
}

uint32_t NB_GetDestIP(void)
{
	uint32_t tmp = 0;
	uint8_t i, byte = 0;
	for (i=0; i< 4; i++)
	{
		MRAM_Read(BSP_NB_DEST_IP_MEMADDR+i,  (uint8_t *)&byte, 1);
		tmp = (tmp << 8) | byte;
	}
	return tmp;
}

void NB_SetDestIP(uint32_t ip)
{
	uint8_t i, byte = 0;
	for (i=0; i< 4; i++)
	{
		byte = (ip >> 24) & 0xff;
		ip <<= 8;
		MRAM_Write(BSP_NB_DEST_IP_MEMADDR+i,  (uint8_t *)&byte, 1);
	}
}

void NB_SetDestPort(uint16_t port)
{
	uint8_t i, byte = 0;
	for (i=0; i< 2; i++)
	{
		byte = (port >> 8) & 0xff;
		port <<= 8;
		MRAM_Write(BSP_NB_DEST_PORT_MEMADDR+i,  (uint8_t *)&byte, 1);
	}
}

uint16_t NB_GetDestPort(void)
{
	uint16_t tmp = 0;
	uint8_t i, byte = 0;
	for (i=0; i< 2; i++)
	{
		MRAM_Read(BSP_NB_DEST_PORT_MEMADDR+i,  (uint8_t *)&byte, 1);
		tmp = (tmp << 8) | byte;
	}
	return tmp;
}
/*
uint32_t NB_GetDestIP(void)
{
	return HAL_RTCEx_BKUPRead(&hrtc, RTC_BCKP_SERVER_IP_ADDRESS);
}


void NB_SetSrcPort(uint16_t port)
{
	uint32_t tmp;
	tmp = 0x0000FFFF & HAL_RTCEx_BKUPRead(&hrtc, RTC_BCKP_UDP_PORTS_ADDRESS);
	tmp |= (port & 0xFFFF) << 16;
	HAL_RTCEx_BKUPWrite(&hrtc, RTC_BCKP_UDP_PORTS_ADDRESS, tmp);
}

uint16_t NB_GetSrcPort(void)
{
	return (uint16_t)((HAL_RTCEx_BKUPRead(&hrtc, RTC_BCKP_UDP_PORTS_ADDRESS) >> 16) & 0xFFFF);
}
*/
#ifdef USE_CMDLINE
unsigned char NB_DBG(tPar *pPar, unsigned char ucParNum, unsigned char ucSwitch)
{
	int b[4] = {0};
	int port=0;
	uint32_t tmp;
	switch(ucSwitch)
	{
		case 0 :
			//NB_SendData(pPar[0].s);
			NB_SendData((uint8_t*)&pPar[0].s, strlen(pPar[0].s), 1);
		break;
		case 1 :
			printf("Command: %d\r\n", NBInfo.Command.Index);
			printf("IMEI: %s\r\n", NBInfo.IMEI);
			printf("ICCID: %s\r\n", NBInfo.ICCID);
			printf("RxLevel=%d\r\n", NBInfo.RxLevel);
			//printf("IMSI: %s\r\n", NBInfo.IMSI);
			printf("IP: %s\r\n", NBInfo.IP);
			printf("ConnectID: %d\r\n", NBInfo.ConnectID);
			tmp = NB_GetDestIP();
			printf("Server=%d.%d.%d.%d:%d\r\n", (int)((tmp >> 24)&0xff), (int)((tmp >> 16)&0xff), (int)((tmp >> 8)&0xff), (int)(tmp&0xff), NB_GetDestPort());
		break;
		case 2 :
			if (strcmp(pPar[0].s, "ip") == 0)
			{
				if (sscanf(pPar[1].s, "%d.%d.%d.%d:%d", &b[3], &b[2], &b[1], &b[0], &port) == 5)
				{
					tmp = ( (b[3]&0xff)<<24 ) | ( (b[2]&0xff)<<16 ) | ( (b[1]&0xff)<<8 ) | ( (b[0]&0xff)<<0 );
					NB_SetDestIP(tmp);
					NB_SetDestPort(port);
					printf( "Saved IP=%d.%d.%d.%d:%d\r\n", (tmp >> 24)&0xff, (tmp >> 16)&0xff, (tmp >> 8)&0xff, tmp&0xff, NB_GetDestPort());
				}
				else printf("Invalid IP syntax: aa.bb.cc.dd:port\r\n");
			}
			else if (strcmp(pPar[0].s, "srcport") == 0)
			{
				if (sscanf(pPar[1].s, "%d", &port) == 1)
				{
					//NB_SetSrcPort(port);
					//printf( "Saved src=%d\r\n", NB_GetSrcPort());
				}
				else printf("Invalid IP syntax: aa.bb.cc.dd:port\r\n");
			}
			else printf("Invalid setting parameters.\r\n");
		break;
	}//switch
return 0;
}
#endif

